var withRequest = flight.component(function () {

  'use strict';

  /**
   * Const
   */
  var baseUrl = 'http://192.168.10.120/ptb/';
  baseUrl = 'https://pflege-tagebuch.de/';
  var fixToken = '533589f3-e0dc-44ae-9b8b-1363c0a80a78';

  /**
   * Module exports
   */

  return withRequest;

  /**
   * Module function
   */

  function withRequest() {
    var requests = [];

	this.defaultAttrs({
		requestDefaultErrorMessage: 'Unknown Error'
	});

	this.createHeaders = function(data){
		settings.headers || {}
		if (data.headers){
			if (data.openMethod){
				var hashObject = this.createHash({mode:'openMethod'});
				var mmAuthPublicHelper = localStorage.getItem('defaultPublicToken');
			} else {
				var hashObject = this.createHash({mode:false});
				var mmAuthPublicHelper = localStorage.getItem('publicToken');
			}
			return {
				'mmAuthTs': hashObject.timestamp,
				'mmAuthPublic':mmAuthPublicHelper,
				'mmAuthAccess':hashObject.accessToken,
				'mmRequestId':data.mmRequestId
			};
		} else {
			return {};
		}
	}

	this.createHash = function(data){
		var timestamp = new Date();

		if (data.mode === 'openMethod'){
			return {
				accessToken : $.sha256(fixToken + localStorage.getItem('defaultPrivateToken') + timestamp.getTime()),
				timestamp: timestamp.getTime()
			};
		} else {
			return {
				accessToken : $.sha256(fixToken + localStorage.getItem('privateToken') + timestamp.getTime()),
				timestamp: timestamp.getTime()
			};
		}
	}

	this.request = function(settings, method) {
		var type = (method || settings.type || 'get').toUpperCase(),
			data = settings.data || {},
			dataType = settings.dataType || 'JSON',
			headers = this.createHeaders({headers: settings.headers, mmRequestId: settings.mmRequestId, openMethod: settings.openMethod}),
			request;

		/*
			"HTML forms (up to HTML version 4 and XHTML 1) only support GET and POST as HTTP request methods.
			A workaround for this is to tunnel other methods through POST by using a hidden form field
			which is read by the server and the request dispatched accordingly."

			http://stackoverflow.com/a/166501/931594
		*/
		if (type !== 'GET' && type !== 'POST') {
			settings._method = type;

			//type = 'POST';
		}

		if (settings.success) {
			settings.success = done.call(this, settings.selector, settings.success);
		}

		if (settings.error) {
			settings.error = fail.call(this, settings.selector, settings.error);
		}

		request = $.ajax($.extend(settings, {
			url: baseUrl + settings.url,
			type: type,
			dataType: dataType,
			data: (type === 'GET' ? '' : data),
			headers: headers,
    	    crossDomain: true,
			processData: false,
    	    contentType: 'application/json; charset=utf-8'
		}));

		if (request && !settings.noAbort) {
			requests.push(request);
		}

		return request;
	}

	function done(selector, action) {
		return function (responseData, textStatus, jqXHR) {
			complete.call(this, jqXHR, selector, action, responseData);
		}.bind(this);
	}

	function fail(selector, action) {
		return function (jqXHR, textStatus) {
			var data = {};

			try {
				if (jqXHR.status !== 0) {
					data = JSON.parse(jqXHR.responseText);
				}
			} catch (e) { }

			data.status = data.status || jqXHR.status || 0;
			data.message = data.statusText || jqXHR.statusText || textStatus;


			// when a ajax request is aborted it is treated as an error
			if (data.status == 0 && jqXHR.statusText === 'abort') {
				data.isAbort = true;
			}

			if (!data.message) {
				data.message = this.attr.requestDefaultErrorMessage;
			}

			complete.call(this, jqXHR, selector, action, data);
		}.bind(this);
	}

    function complete(request, selector, action, responseData) {
    	removeRequest(request);
    	if (typeof action == 'function') {
    		action(responseData);
    		return;
    	}
    	this.trigger(selector, action, responseData);
    }

    function removeRequest(request) {
    	requests.splice(requests.indexOf(request), 1);
    }

    // mixin methods

    this.get = function (evt, settings) {
    	this.request(settings, "GET");
    };

    this.post = function (evt, settings) {
    	this.request(settings, "POST");
    };

    this.put = function (evt, settings) {
    	return this.request(settings, "PUT");
    };

    this.destroy = this['delete'] = function (evt, settings) {
    	return this.request(settings, "DELETE");
    };

    this.abort = function (evt, request) {
    	if (request && typeof request.abort == 'function') {
    		removeRequest(request);
    		request.abort();
    	}
    };

    this.abortAllRequests = function () {
    	requests.forEach(this.abort, null);
    };

    this.after('initialize', function () {
    	var thisComponent = this;
    	this.on(window, 'unload', this.abortAllRequests);

    	this.on('dataDoPostRequest', this.post);
    	this.on('dataDoGetRequest', this.get);
    	this.on('dataDoPutRequest', this.put);
    	this.on('dataDoDestroyRequest', this.destroy);
    	this.on('dataDoDeleteRequest', this.destroy);
    	this.on('dataAbortRequest', this.abort);
    	this.on('dataAbortAllRequests', this.abortAllRequests);
    	this.on('dataDoGetExport', function(evt, data){
    		$('#export').trigger('uiProcessComplete');
    		window.requestFileSystem(LocalFileSystem.PERSISTENT, 0, function(fs) {

	    		var fileTransfer = new FileTransfer();
	    		var uri = encodeURI(data.Export.pdf_download_url);

	    		fileTransfer.download(
	    		    uri,
	    		    fs.root.toURL() + "/Pflegetagebuch_Report" + ".pdf",
	    		    function(entry) {
	    		    	window.plugins.socialsharing.share('', 'Pflegetagebuch Report', entry.nativeURL);


	    		    },
	    		    function(error) {
	    		    	console.log(error);
	    		        console.log("download error source " + error.source);
	    		        console.log("download error target " + error.target);
	    		        console.log("upload error code" + error.code);
	    		    },
	    		    false,
	    		    {
	    		        headers: thisComponent.createHeaders({headers:true})
	    		    }
	    		)
	    	});
    	});

    });
  }

}());
